#region Includes
using System;
using System.Collections.Generic;
using System.Net;
using System.Windows.Forms;
using System.IO;
using KellermanSoftware.NetSFtpLibrary;
using KellermanSoftware.NetSFtpLibrary.Interfaces;
using Microsoft.Win32;
#endregion

namespace KellermanSoftware.NetSftpLibraryDemo
{


    public partial class Demo : Form
    {
        #region Class Variables
        private SFTP _sftp;
        
        string _currentLocalDirectory;
        bool _loading;
        #endregion

        #region Constructor
        public Demo()
        {
            InitializeComponent();
            cboProxyType.SelectedIndex = 0;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Get a remote directory listing and display it in the list box
        /// </summary>
        /// <param name="dirName">The sub directory to load, string.empty to reload the current directory</param>
        private void LoadRemoteDirectory(String dirName)
        {
            if (_loading)
            {
                return;
            }

            _loading = true;
            Cursor = Cursors.WaitCursor;

            //string myString = _sftp.GetRawDirectoryListing();

            if (dirName == "..")
            {
                _sftp.CurrentDirectory = _sftp.ParentDirectory;
            }
            else
            {
                _sftp.CurrentDirectory += dirName;
            }

            while (_sftp.IsBusy)
            {
                System.Threading.Thread.Sleep(100);
            }

            List<FTPFileInfo> files;
            try
            {
                files = _sftp.GetDirectoryListing();
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }

            listRemote.Items.Clear();
            ListViewItem lvi = new ListViewItem("..");
            lvi.ImageIndex = 1;
            listRemote.Items.Add(lvi);

            foreach (FTPFileInfo file in files)
            {
                if (file.IsDirectory)
                {
                    if (file.FileName != ".." && file.FileName != ".")
                    {
                        lvi = new ListViewItem(file.FileName);
                        lvi.ImageIndex= 0;
                        listRemote.Items.Add(lvi);
                    }
                }
            }

            foreach (FTPFileInfo file in files)
            {
                if (!file.IsDirectory)
                {
                    lvi = new ListViewItem(file.FileName);
                    lvi.SubItems.Add(file.SizeInBytes.ToString());
                    lvi.SubItems.Add(file.Modified.ToString());
                    listRemote.Items.Add(lvi);
                }
            }

            if(_sftp.CurrentDirectory != string.Empty && !cmboRemotePath.Items.Contains('/' + _sftp.CurrentDirectory))
                cmboRemotePath.Items.Add(_sftp.CurrentDirectory);
            cmboRemotePath.SelectedItem = _sftp.CurrentDirectory;

            Cursor = Cursors.Default;
            _loading = false;
        }

        /// <summary>
        /// Property to store the current path of the display local directory
        /// </summary>
        private string CurrentLocalDirectory
        {
            get 
            {
                return _currentLocalDirectory;
            }
            set 
            {
                if (!value.EndsWith(new String(Path.DirectorySeparatorChar, 1)))
                    value += Path.DirectorySeparatorChar;
                _currentLocalDirectory = value;
            }
        }

        /// <summary>
        /// Get a local directory listing and display it in the list box
        /// </summary>
        /// <param name="dirName">The sub directory to load, string.empty to reload the current directory</param>
        private void LoadLocalDirectory(String dirName)
        {
            if (dirName == "..")
            {
                if (CurrentLocalDirectory.Length > 1)
                {
                    try
                    {
                        DirectoryInfo d = Directory.GetParent(CurrentLocalDirectory);
                        if (d != null && d.Parent != null)
                            CurrentLocalDirectory = d.Parent.FullName;
                    }
                    catch (WebException ex)
                    {
                        MessageBox.Show("Web error:" + ex.Message);
                        return;
                    }
                }
            }
            else
                CurrentLocalDirectory += dirName;

            
            listLocal.Items.Clear();
            ListViewItem lvi = new ListViewItem("..");
            lvi.ImageIndex = 1;
            listLocal.Items.Add(lvi);

            string[] dirs = Directory.GetDirectories(CurrentLocalDirectory);
            foreach (String dir in dirs)
            {
                DirectoryInfo d = new DirectoryInfo(dir);
                lvi = new ListViewItem(d.Name);
                lvi.ImageIndex = 0;
                listLocal.Items.Add(lvi);
            }

            DirectoryInfo curDir = new DirectoryInfo(CurrentLocalDirectory);
            foreach (FileInfo file in curDir.GetFiles())
            {
                lvi = new ListViewItem(file.Name);
                lvi.SubItems.Add(file.Length.ToString());
                lvi.SubItems.Add(file.LastWriteTimeUtc.ToString());
                listLocal.Items.Add(lvi);
            }
        }


         
        /// <summary>
        /// Connects to the SFTP server
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            if (groupLogin.Enabled)
            {         
                _sftp= new SFTP();
                _sftp.HostAddress = txtHost.Text;
                _sftp.OverwriteFiles = chkOverWrite.Checked;
                _sftp.OverwriteReadOnlyFiles = chkReadOnly.Checked;
                _sftp.Password = txtPassword.Text;
                _sftp.Port = Int32.Parse(txtPort.Text);
                _sftp.ReadWriteTimeout = Int32.Parse(txtReadWriteTimeout.Text);
                _sftp.UserName = txtUserName.Text;
                _sftp.SshKeyFile = txtPrivateKey.Text;
                _sftp.SshPassphrase = txtPassphrase.Text;

                _sftp.Proxy.Type = (ProxyType) Enum.Parse(typeof(ProxyType), cboProxyType.SelectedItem.ToString(), true);
                _sftp.Proxy.Host = txtProxyHost.Text;
                _sftp.Proxy.UserName = txtProxyUserName.Text;
                _sftp.Proxy.Password = txtProxyPassword.Text;
                _sftp.Proxy.Port = Convert.ToInt32(txtProxyPort.Text);

                if (txtFileLog.Text.Length > 0)
                    _sftp.EnableLogging(txtFileLog.Text);

                try
                {
                    _sftp.Connect();

                    groupLogin.Enabled = false;
                    groupOptions.Enabled = false;
                    btnConnect.Text = "Disconnect";
                    btnConnect.ImageKey = "disconnect";
                    ftpSplitContainer.Enabled = true;

                    LoadRemoteDirectory(string.Empty);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "SFTP Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            else
            {
                _sftp.Disconnect();
                _sftp.Dispose();
                groupLogin.Enabled = true;
                groupOptions.Enabled = true;
                btnConnect.ImageKey = "connect";
                btnConnect.Text = "Connect";
                ftpSplitContainer.Enabled = false;
            }
            Cursor = Cursors.Default;
        }

        /// <summary>
        /// Fired when the form is loaded, initializes the SFTP component and the GUI
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                _sftp = new SFTP();

                txtReadWriteTimeout.Text = _sftp.ReadWriteTimeout.ToString();

                // load the drive list box
                foreach (DriveInfo drive in DriveInfo.GetDrives())
                    if (drive.Name == @"C:\")
                    {
                        int i = cmboDrive.Items.Add(drive.Name);
                        cmboDrive.SelectedIndex = i;
                    }
                    else
                        cmboDrive.Items.Add(drive.Name);


                // load the local directory
                CurrentLocalDirectory = @"C:\";
                LoadLocalDirectory(string.Empty);

                //Get defaults from registry
                txtHost.Text= GetReg("KellermanSoftware", "Kellerman SFTP", "Host");
                txtUserName.Text =GetReg("KellermanSoftware", "Kellerman SFTP", "Login");
            }
            catch (Exception myException)
            {
                MessageBox.Show(myException.Message);
                return;
            }
        }

        /// <summary>
        /// Uploads selected file(s) and directory(ies)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpload_Click(object sender, EventArgs e)
        {
            try
            {
                for (int i = 0; i < listLocal.SelectedItems.Count; i++)
                {
                    if (listLocal.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listLocal.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        if (_sftp.FileExists(listLocal.SelectedItems[i].Text))
                        {
                            if (MessageBox.Show("File already exists! If you cancelled the upload and would like to resume it, press yes. If you did not cancel the upload, press no.", "Resume Upload", MessageBoxButtons.YesNo) == DialogResult.Yes)
                            {
                                using (ProgressForm statusDialog = new ProgressForm())
                                {
                                    statusDialog.Sftp = _sftp;
                                    statusDialog.IsDownloading = false;
                                    _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                                    _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;
                                    _sftp.ResumeUploadFileAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _sftp.CurrentDirectory + listLocal.SelectedItems[i].Text);
                                    statusDialog.ShowDialog();
                                    _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                                    _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                                }
                                LoadRemoteDirectory(string.Empty);
                            }
                            return;
                        }

                        using (ProgressForm statusDialog = new ProgressForm())
                        {
                            statusDialog.Sftp = _sftp;
                            statusDialog.IsDownloading = false;
                            _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                            _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;
                            _sftp.UploadFileAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _sftp.CurrentDirectory + listLocal.SelectedItems[i].Text);
                            statusDialog.ShowDialog();
                            _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                            _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                        }
                        LoadRemoteDirectory(string.Empty);
                    }
                    else
                    {
                        using (ProgressForm statusDialog = new ProgressForm())
                        {
                            statusDialog.Sftp = _sftp;
                            statusDialog.IsDownloading = false;
                            _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                            _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;

                            _sftp.UploadDirectoryAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text);

                            statusDialog.ShowDialog();
                            _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                            _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                        }
                        LoadRemoteDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
        }

        /// <summary>
        /// If a folder, or the up folder icon, is double clicked it is navigated to
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listRemote_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                if (listRemote.SelectedItems.Count == 0 || listRemote.SelectedItems[0].ImageIndex == (int) ImageType.File)
                    return;
                LoadRemoteDirectory(listRemote.SelectedItems[0].Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Creates a remote folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteNewFolder_Click(object sender, EventArgs e)
        {
            listRemote.LabelEdit = true;
            ListViewItem lvi = listRemote.Items.Add("New Folder");
            lvi.ImageIndex = 0;
            lvi.BeginEdit();     
        }

        /// <summary>
        /// Deletes the selected remote file/folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteDelete_Click(object sender, EventArgs e)
        {
            if (listRemote.SelectedItems.Count == 0 || listRemote.SelectedItems[0].Text == "..")
                return;
            try
            {
                if (listRemote.SelectedItems[0].ImageIndex == (int) ImageType.Directory)
                {
                    _sftp.DeleteDirectory(listRemote.SelectedItems[0].Text);
                }
                else
                {
                    _sftp.DeleteFile(listRemote.SelectedItems[0].Text);
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            LoadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Refreshes the remote directory listing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteRefresh_Click(object sender, EventArgs e)
        {
            LoadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Fired after the user is done entering a new/renaming a folder and file
        /// It validates the name is valid and then updates it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listRemote_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            e.CancelEdit = true;
            if (string.IsNullOrEmpty(e.Label))
                return;
            try
            {
                if (listRemote.Items[e.Item].ImageIndex == (int) ImageType.Directory)
                {
                    if (listRemote.Items[e.Item].Text == "New Folder")
                    {
                        if (_sftp.CreateDirectory(e.Label))
                        {
                            e.CancelEdit = false;
                        }
                        else
                        {
                            MessageBox.Show("Directory creation failed.");
                        }
                    }
                    else
                    {
                        if (_sftp.RenameDirectory(listRemote.Items[e.Item].Text, e.Label))
                        {
                            e.CancelEdit = false;
                        }
                        else
                        {
                            MessageBox.Show("Directory rename failed.");
                        }
                    }
                }
                else
                {
                    if (_sftp.RenameFile(listRemote.Items[e.Item].Text, e.Label))
                    {
                        e.CancelEdit = false;
                    }
                    else
                    {
                        MessageBox.Show("File rename failed.");
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            catch (Exception)
            {
                return;
            }

            LoadRemoteDirectory(string.Empty);
        }

        /// <summary>
        /// Renames the selected folder/file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRemoteRename_Click(object sender, EventArgs e)
        {
            if (listRemote.SelectedItems.Count == 0 || listRemote.SelectedItems[0].Text == "..")
                return;

            listRemote.LabelEdit = true;
            listRemote.SelectedItems[0].BeginEdit();
        }







        /// <summary>
        /// If a folder, or the up folder icon, is double clicked it is navigated to
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listLocal_DoubleClick(object sender, EventArgs e)
        {
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].ImageIndex == (int) ImageType.File)
                return;
            LoadLocalDirectory(listLocal.SelectedItems[0].Text);           
        }

        /// <summary>
        /// Fired after the user is done entering a new/renaming a folder and file
        /// It validates the name is valid and then updates it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void listLocal_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            e.CancelEdit = true;
            if (string.IsNullOrEmpty(e.Label))
                return;

            if (listLocal.Items[e.Item].ImageIndex == (int) ImageType.Directory)
            {
                if (listLocal.Items[e.Item].Text == "New Folder")
                {
                    try
                    {
                        if (!Directory.Exists(CurrentLocalDirectory + e.Label))
                        {
                            Directory.CreateDirectory(CurrentLocalDirectory + e.Label);
                            e.CancelEdit = false;
                        }else
                            MessageBox.Show("Directory creation failed.");
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Directory creation failed.");
                    }     
                }
                else
                {
                    try
                    {
                        if (!Directory.Exists(CurrentLocalDirectory + e.Label))
                        {
                            Directory.Move(CurrentLocalDirectory + listLocal.Items[e.Item].Text, CurrentLocalDirectory + e.Label);
                            e.CancelEdit = false;
                        }else
                            MessageBox.Show("Directory rename failed.");
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Directory rename failed.");
                    }    
                }
            }
            else
            {
                try
                {
                    if (!File.Exists(CurrentLocalDirectory + e.Label))
                    {
                        File.Move(CurrentLocalDirectory + listLocal.Items[e.Item].Text, CurrentLocalDirectory + e.Label);
                        e.CancelEdit = false;
                    }
                    else
                        MessageBox.Show("File rename failed.");
                }
                catch (Exception)
                {
                    MessageBox.Show("File rename failed.");
                }    
            }

            LoadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Creates a new local folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalNewFolder_Click(object sender, EventArgs e)
        {
            listLocal.LabelEdit = true;
            ListViewItem lvi = listLocal.Items.Add("New Folder");
            lvi.ImageIndex = 0;
            lvi.BeginEdit();  
        }

        /// <summary>
        /// Deletes the selected folder/file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalDelete_Click(object sender, EventArgs e)
        {
            listLocal.LabelEdit = false;
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].Text == "..")
                return;
            try
            {
                string f = CurrentLocalDirectory + listLocal.SelectedItems[0].Text;
                if (listLocal.SelectedItems[0].ImageIndex == (int) ImageType.Directory)
                {
                    Directory.Delete(f, true);
                }
                else
                {
                    File.Delete(f);
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            
            
            LoadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Refreshes the current local directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalRefresh_Click(object sender, EventArgs e)
        {
            LoadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Renames the currently selected file/folder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnLocalRename_Click(object sender, EventArgs e)
        {
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].Text == "..")
                return;

            listLocal.LabelEdit = true;
            listLocal.SelectedItems[0].BeginEdit();
        }

        /// <summary>
        /// Downloads the currently selected remote files/folders
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDownload_Click(object sender, EventArgs e)
        {
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        ProgressForm statusDialog;

                        if (File.Exists(CurrentLocalDirectory + listRemote.SelectedItems[i].Text))
                        {
                            if (MessageBox.Show("File already exists! If you cancelled the download and would like to resume it, press yes. If you did not cancel the download, press no.", "Resume Download", MessageBoxButtons.YesNo) == DialogResult.Yes)
                            {
                                statusDialog = new ProgressForm();
                                statusDialog.Sftp = _sftp;
                                statusDialog.IsDownloading = true;

                                _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                                _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;
                                _sftp.ResumeDownloadFileAsync(CurrentLocalDirectory + listRemote.SelectedItems[i].Text, listRemote.SelectedItems[i].Text);
                                if (!statusDialog.IsDisposed)
                                    statusDialog.ShowDialog();
                                _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                                _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                                LoadLocalDirectory(string.Empty);
                            }
                            return;
                        }

                        statusDialog = new ProgressForm();
                        statusDialog.Sftp = _sftp;
                        statusDialog.IsDownloading = true;

                        _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;
                        _sftp.DownloadFileAsync(CurrentLocalDirectory + listRemote.SelectedItems[i].Text, listRemote.SelectedItems[i].Text);
                        if (!statusDialog.IsDisposed)
                            statusDialog.ShowDialog();
                        _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                    }
                    else
                    {
                        ProgressForm statusDialog = new ProgressForm();
                        statusDialog.Sftp = _sftp;
                        statusDialog.IsDownloading = true;

                        _sftp.OverallProgressEvent += statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;

                        _sftp.DownloadDirectoryAsync(CurrentLocalDirectory, listRemote.SelectedItems[i].Text);

                        if (!statusDialog.IsDisposed)
                            statusDialog.ShowDialog();
                        _sftp.OverallProgressEvent -= statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                    }
                }
                LoadLocalDirectory(string.Empty);
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
        }

        /// <summary>
        /// Syncs the current local and remote directories
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSync_Click(object sender, EventArgs e)
        {
            try
            {
                for (int i = 0; i < listLocal.SelectedItems.Count; i++)
                {
                    if (listLocal.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listLocal.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        ProgressForm statusDialog = new ProgressForm();
                        statusDialog.Sftp = _sftp;
                        statusDialog.IsDownloading = false;
                        _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;
                        _sftp.UploadFileAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _sftp.CurrentDirectory + listLocal.SelectedItems[i].Text);
                        statusDialog.ShowDialog();
                        _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                        LoadRemoteDirectory(string.Empty);
                    }
                    else
                    {
                        ProgressForm statusDialog = new ProgressForm();
                        statusDialog.Sftp = _sftp;
                        statusDialog.IsDownloading = false;
                        _sftp.FileProgressEvent += statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent += statusDialog.FtpTransferCompleteEvent;

                        _sftp.SynchronizeUploadAsync(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text);

                        statusDialog.ShowDialog();
                        _sftp.FileProgressEvent -= statusDialog.FtpProgressEvent;
                        _sftp.TransferCompleteEvent -= statusDialog.FtpTransferCompleteEvent;
                        LoadRemoteDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
        }

        /// <summary>
        /// Changes the local directories current drive
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmboDrive_SelectedIndexChanged(object sender, EventArgs e)
        {
            CurrentLocalDirectory = cmboDrive.Text;
            LoadLocalDirectory(string.Empty);
        }

        /// <summary>
        /// Loads a remote previously viewed remote directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmboRemotePath_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_loading == false)
            {
                _sftp.CurrentDirectory = "";
                LoadRemoteDirectory((string)cmboRemotePath.SelectedItem);
            }
        }

        /// <summary>
        /// Displays the file size of the selected remote file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFileSize_Click(object sender, EventArgs e)
        {
            bool shown = false;
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        MessageBox.Show(_sftp.GetFileSize(listRemote.SelectedItems[i].Text).ToString());
                        shown = true;
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }

            if (!shown)
                MessageBox.Show("Please select a file!");
        }

        /// <summary>
        /// Displays the directory size of the selected remote directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDirSize_Click(object sender, EventArgs e)
        {
            bool shown = false;
            Cursor = Cursors.WaitCursor; 
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.Directory)
                    {
                        MessageBox.Show(_sftp.GetDirectorySize(listRemote.SelectedItems[i].Text).ToString());
                        shown = true;
                    }
                }

                if (!shown)
                    MessageBox.Show("Please select a directory!");
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
            }

            Cursor = Cursors.Default; 

        }

        /// <summary>
        /// Displays the file modification date of the selected remote file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFileDate_Click(object sender, EventArgs e)
        {
            bool shown = false;
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        MessageBox.Show(_sftp.GetFileDate(listRemote.SelectedItems[i].Text).ToString());
                        shown = true;
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            if (!shown)
                MessageBox.Show("Please select a file!");
        }

        /// <summary>
        /// Update the file with a unique name
        /// The name it is saved as will be displayed in a message box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUploadUnique_Click(object sender, EventArgs e)
        {
            if (listLocal.SelectedItems.Count == 0 || listLocal.SelectedItems[0].Text == "..")
                return;

            Cursor = Cursors.WaitCursor; 

            try
            {
                String name = _sftp.UploadFileUnique(CurrentLocalDirectory + listLocal.SelectedItems[0].Text, listLocal.SelectedItems[0].Text);                
                LoadRemoteDirectory(string.Empty);
                MessageBox.Show("Uploaded file as:" + name);
            }           
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
            }

            Cursor = Cursors.Default; 
        }

        /// <summary>
        /// Upload a file/directory syncronously
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSyncUpload_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor; 
            try
            {

                for (int i = 0; i < listLocal.SelectedItems.Count; i++)
                {
                    if (listLocal.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listLocal.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        if (_sftp.FileExists(listLocal.SelectedItems[i].Text))
                        {
                            MessageBox.Show("File already exists!");
                            return;
                        }
                        _sftp.UploadFile(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, _sftp.CurrentDirectory + listLocal.SelectedItems[i].Text);
                        LoadRemoteDirectory(string.Empty);
                    }
                    else
                    {
                        _sftp.UploadDirectory(CurrentLocalDirectory + listLocal.SelectedItems[i].Text, listLocal.SelectedItems[i].Text);
                        LoadRemoteDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            Cursor = Cursors.Default; 
        }

        /// <summary>
        /// Delete all .txt files in the current directory
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDeleteWildcard_Click(object sender, EventArgs e)
        {
            try
            {
                _sftp.DeleteByWildcard("*.txt", false);
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
                return;
            }
            LoadRemoteDirectory(string.Empty);
        }


        private void btnDownloadSync_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            try
            {
                for (int i = 0; i < listRemote.SelectedItems.Count; i++)
                {
                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.Parent)
                        continue;

                    if (listRemote.SelectedItems[i].ImageIndex == (int) ImageType.File)
                    {
                        _sftp.DownloadFile(CurrentLocalDirectory + listRemote.SelectedItems[i].Text, _sftp.CurrentDirectory + listRemote.SelectedItems[i].Text);
                        LoadLocalDirectory(string.Empty);
                    }
                    else
                    {
                        _sftp.DownloadDirectory(CurrentLocalDirectory, listRemote.SelectedItems[i].Text);
                        LoadLocalDirectory(string.Empty);
                    }
                }
            }
            catch (WebException ex)
            {
                MessageBox.Show("Web error:" + ex.Message);
            }
            Cursor = Cursors.Default; 
        }

        /// <summary>
        /// Save a key to the registry
        /// </summary>
        /// <param name="sCompanyName">Company Name</param>
        /// <param name="sProgramName">Program Name</param>
        /// <param name="sValue">Value to set</param>
        /// <param name="sSetting">The setting of the value</param>
        public static void SaveReg(string sCompanyName, string sProgramName, string sValue, string sSetting)
        {
            string sKey = "SOFTWARE\\" + sCompanyName + "\\" + sProgramName;
            RegistryKey oKey = Registry.CurrentUser.CreateSubKey(sKey);
            if (oKey != null) oKey.SetValue(sValue, sSetting);
        }

        /// <summary>
        /// Get a registry key.
        /// </summary>
        /// <param name="sCompanyName">Company Name</param>
        /// <param name="sProgramName">Program Name</param>
        /// <param name="sValue">Registry value to look for.</param>
        /// <returns>Registry key</returns>
        public static String GetReg(string sCompanyName, string sProgramName, string sValue)
        {
            string sResult = string.Empty;

            string sKey = "SOFTWARE\\" + sCompanyName + "\\" + sProgramName;

            RegistryKey oKey = Registry.CurrentUser.OpenSubKey(sKey, false);

            if (oKey != null)
                sResult = oKey.GetValue(sValue, "").ToString();

            return sResult;

        }

        private void TestTransfer_FormClosing(object sender, FormClosingEventArgs e)
        {
            _sftp.Disconnect();
            SaveReg("KellermanSoftware", "Kellerman SFTP", "Host", txtHost.Text);
            SaveReg("KellermanSoftware", "Kellerman SFTP", "Login", txtUserName.Text);
        }


        private void btnPrivateKey_Click(object sender, EventArgs e)
        {
            if (fileDialog.ShowDialog(this) == DialogResult.OK)
                txtPrivateKey.Text = fileDialog.FileName;
        }
        #endregion









    }
}