#region Includes
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using KellermanSoftware.NetSFtpLibrary;
#endregion

namespace NetSFtpLibrarySimpleDemo
{
    public partial class Demo : Form
    {
        #region Class Variables
        private SFTP _sftp;
        private List<FTPFileInfo> _files;
        private delegate void SetStatusDelegate(string text);
        private delegate void LoadDirectoryDelegate(string path);
        private delegate void SetCommandGuiDelegate(bool enabled);
        #endregion

        #region Constructor
        public Demo()
        {
            InitializeComponent();
        }
        #endregion

        #region Methods

        /// <summary>
        /// Show an error message box
        /// </summary>
        /// <param name="myException"></param>
        private void ShowError(Exception myException)
        {
            MessageBox.Show(myException.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        /// <summary>
        /// Event that is fired when a transfer completes
        /// </summary>
        private void FtpTransferCompleted(Object sender, TransferCompletedEventArgs e)
        {
            SetStatus("File transfer completed...");
            LoadDirectory(string.Empty);
            SetCommandGui(true);
        }

        /// <summary>
        /// Event that is fired when a transfer is in progress
        /// </summary>
        private void FtpTransferProgressChangedEvent(Object sender, TransferProgressChangedEventArgs e)
        {
            try
            {
                SetStatus("Transfer [" + _sftp.CurrentFileBeingTransferred + " in " + _sftp.CurrentDirectoryBeingTransferred + "]: " + e.ProgressPercentage + "% (" + e.BytesTransferred + " of " + e.TotalBytesToTransfer + " bytes)");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Fired when form loads: initializes the FTPTransfer class and sets the event handlers
        /// </summary>
        private void Demo_Load(object sender, EventArgs e)
        {
            try
            {
                _sftp = new SFTP();
                _sftp.EnableLogging();
                _sftp.OverallProgressEvent += FtpTransferProgressChangedEvent;
                _sftp.FileProgressEvent += FtpTransferProgressChangedEvent;
                _sftp.TransferCompleteEvent += FtpTransferCompleted;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Either enables or disables the connection GUI
        /// </summary>
        private void SetConnectGui(bool enabled)
        {
            txtHost.Enabled = enabled;
            txtLogin.Enabled = enabled;
            txtPass.Enabled = enabled;
            txtPort.Enabled = enabled;
            txtPrivateKeyFile.Enabled = enabled;
            txtPassphrase.Enabled = enabled;
            chkOverWrite.Enabled = enabled;
            chkReadOnly.Enabled = enabled;
        }

        /// <summary>
        /// Either enables or disables the FTP command GUI
        /// </summary>
        private void SetCommandGui(bool enabled)
        {
            // this gets called from an event, so make sure this is on the right thread
            if (InvokeRequired)
            {
                // this is not on the right thread, so invoke it on the correct one
                SetCommandGuiDelegate setCommandGuiProc = SetCommandGui;
                Invoke(setCommandGuiProc, new object[] { enabled });
                return;
            }

            btnCreateDirectory.Enabled = enabled;
            btnDeleteDirectory.Enabled = enabled;
            btnDownloadDir.Enabled = enabled;
            btnUploadDir.Enabled = enabled;

            btnDownloadFile.Enabled = enabled;
            btnUploadFile.Enabled = enabled;
            btnDeleteFile.Enabled = enabled;
            btnRenameFile.Enabled = enabled;
        }

        /// <summary>
        /// Set the status bar text
        /// </summary>
        private void SetStatus(String s)
        {
            // this gets called from an event, so make sure this is on the right thread
            if (status.InvokeRequired)
            {
                // this is not on the right thread, so invoke it on the correct one
                SetStatusDelegate setStatusProc = SetStatus;
                status.Invoke(setStatusProc, new object[] { s });
                return;
            }
            status.Items["Label"].Text = s;
            status.Refresh();
        }

        /// <summary>
        /// Set the status bar text to the status code of an FTP Request
        /// </summary>
        private void SetCode(FtpResponse response)
        {
            status.Items["FTPCode"].Text = "FTP Status: " + response.Text;
            status.Refresh();
        }

        /// <summary>
        /// Load a directory listing into the GUI
        /// </summary>
        /// <param name="dirName">The directory to load, pass string.empty to reload the current directory</param>
        private void LoadDirectory(String dirName)
        {
            SetStatus("Loading directory listing...");

            // this gets called from an event, so make sure this is on the right thread
            if (treeDir.InvokeRequired)
            {
                // this is not on the right thread, so invoke it on the correct one
                LoadDirectoryDelegate loadDirectoryProc = LoadDirectory;
                Invoke(loadDirectoryProc, new object[] { dirName });
                return;
            }


            if (dirName.Length > 0)
                _sftp.CurrentDirectory = _sftp.CurrentDirectory + dirName;

            _files = _sftp.GetDirectoryListing();
            TreeNode n;

            treeDir.Nodes.Clear();
            treeFiles.Nodes.Clear();
            bool upDir = false;

            foreach (FTPFileInfo file in _files)
            {
                if (file.IsDirectory)
                {
                    treeDir.Nodes.Add(file.FileName);
                    if (file.FileName == ".." || file.FileName == ".")
                        upDir = true;
                }
                else
                {
                    n = treeFiles.Nodes.Add(file.FileName);
                    n.ToolTipText = "Modified: " + file.Modified + "\nSize: " + file.SizeInBytes;
                }
            }

            if(!upDir)
                treeDir.Nodes.Insert(0, "..");

            SetStatus("");
        }

        /// <summary>
        /// Event that is fired when a the connect button is clicked, set the FTPTransfer properties and load the default directory
        /// </summary>
        private void btnConnect_Click(object sender, EventArgs e)
        {
            try
            {
                btnConnect.Enabled = false;
                if (btnConnect.Text == "Go")
                {
                    SetStatus("Logging in...");
                    SetConnectGui(false);
                    _sftp.HostAddress = txtHost.Text;
                    _sftp.UserName = txtLogin.Text;
                    _sftp.Password = txtPass.Text;
                    _sftp.Port = Int32.Parse(txtPort.Text);
                    _sftp.OverwriteFiles = chkOverWrite.Checked;
                    _sftp.OverwriteReadOnlyFiles = chkReadOnly.Checked;
                    _sftp.SshKeyFile = txtPrivateKeyFile.Text;
                    _sftp.SshPassphrase = txtPassphrase.Text;
                    _sftp.Connect();

                    LoadDirectory(string.Empty);
                    btnConnect.Text = "Dispose";
                    SetCommandGui(true);
                }
                else
                {
                    _sftp.Disconnect();
                    _sftp.Dispose();
                    treeDir.Nodes.Clear();
                    treeFiles.Nodes.Clear();
                    btnConnect.Text = "Go";
                    SetConnectGui(true);
                    SetCommandGui(false);
                }
                btnConnect.Enabled = true;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Delete a selected directory
        /// </summary>
        private void btnDeleteDirectory_Click(object sender, EventArgs e)
        {
            try
            {
                if (treeDir.SelectedNode != null)
            {
                if (treeDir.SelectedNode.Text != "." && treeDir.SelectedNode.Text != "..")
                {
                    SetStatus("Deleting directory...");
                    if (_sftp.DeleteDirectory(treeDir.SelectedNode.Text))
                    {
                        SetStatus("");
                        LoadDirectory(string.Empty);
                        SetCode(_sftp.LastFtpResponse);
                    }
                    else
                        SetStatus("Directory deletion failed...");
                }
            }
            else
                MessageBox.Show("Please select a directory to delete!");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Delete a selected file
        /// </summary>
        private void btnDeleteFile_Click(object sender, EventArgs e)
        {
            try
            {
                if (treeFiles.SelectedNode != null)
                {
                    SetStatus("Deleting file...");
                    _sftp.DeleteFile(treeFiles.SelectedNode.Text);
                    LoadDirectory(string.Empty);
                    SetCode(_sftp.LastFtpResponse);
                }
                else
                    MessageBox.Show("Please select a file to delete!");
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Fired when the directory tree is double clicked, navigate to that directory
        /// </summary>
        private void treeDir_DoubleClick(object sender, EventArgs e)
        {
            if(treeDir.SelectedNode != null) LoadDirectory(treeDir.SelectedNode.Text);
        }

        /// <summary>
        /// Rename a selected file by placing an edit box on it
        /// </summary>
        private void btnRenameFile_Click(object sender, EventArgs e)
        {
            if (treeFiles.SelectedNode == null)
            {
                MessageBox.Show("Please select a file to rename!");
                return;
            }

            treeFiles.LabelEdit = true;
            if (treeFiles.SelectedNode.IsEditing == false)
                treeFiles.SelectedNode.BeginEdit();

        }

        /// <summary>
        /// Fired when the user is done editing the file's name, update it
        /// </summary>
        private void treeFiles_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            try
            {
                if (e.Label != null)
                {
                    if(e.Label.Length > 0)
                    {
                        SetStatus("Checking for file existence...");
                        if (_sftp.FileExists(e.Label))
                        {
                            MessageBox.Show("File already exists!");
                            e.CancelEdit = true;
                        }
                        else
                        {
                            SetStatus("Renaming file...");
                            _sftp.RenameFile(e.Node.Text, e.Label);
                            SetCode(_sftp.LastFtpResponse);
                        }
                                        
                        SetStatus("");
                    }
                    else
                    {
                        MessageBox.Show("File name's cannot be blank!");
                        e.CancelEdit = true;
                    }
                }
                treeFiles.LabelEdit = false;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Fired when the user is done editing the new directory's name, create it
        /// </summary>
        private void treeDir_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            try
            {
                if (e.Label != null)
                {
                    if (e.Label.Length > 0)
                    {
                        bool exists = false;
                        foreach (TreeNode n in treeDir.Nodes)
                            if (n.Text == e.Label && !n.IsEditing)
                            {
                                exists = true;
                                break;
                            }

                        if (exists)
                        {
                            e.CancelEdit = true;
                            e.Node.Remove();
                            MessageBox.Show("Directory already exists!");
                        }
                        else
                        {
                            SetStatus("Creating directory...");
                            _sftp.CreateDirectory(e.Label);
                            SetCode(_sftp.LastFtpResponse);
                        }

                        SetStatus("");
                    }
                    else
                    {
                        MessageBox.Show("Directory name cannot be blank!");
                        e.CancelEdit = true;
                    }
                }
                treeFiles.LabelEdit = false;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Upload a file
        /// </summary>
        private void btnUploadFile_Click(object sender, EventArgs e)
        {
            try
            {
                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    // check for file existence
                    String fileName = Path.GetFileName(openFileDialog.FileName);
                    if (!_sftp.FileExists(fileName))
                    {
                        SetCommandGui(false);
                        _sftp.UploadFileAsync(openFileDialog.FileName, fileName);
                    }
                    else
                    {
                        MessageBox.Show("File already exists!");
                    }
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Download a file
        /// </summary>
        private void btnDownloadFile_Click(object sender, EventArgs e)
        {
            try
            {
                if (treeFiles.SelectedNode == null)
                {
                    SetCommandGui(false);
                    MessageBox.Show("Please select a file to download!");
                    return;
                }

                saveFileDialog.FileName = treeFiles.SelectedNode.Text;
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    _sftp.DownloadFileAsync(saveFileDialog.FileName, treeFiles.SelectedNode.Text);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Begin creating a directory by creating a new edit box for the name
        /// </summary>
        private void btnCreateDirectory_Click(object sender, EventArgs e)
        {
            treeDir.LabelEdit = true;
            TreeNode n = treeDir.Nodes.Add("New Directory", "New Directory");
            treeDir.SelectedNode = n;
            n.BeginEdit();
        }

        /// <summary>
        /// Upload a directory
        /// </summary>
        private void btnUploadDir_Click(object sender, EventArgs e)
        {
            try
            {
                if (folderBrowser.ShowDialog() == DialogResult.OK)
                {
                    SetCommandGui(false);
                    DirectoryInfo d = new DirectoryInfo(folderBrowser.SelectedPath);
                    _sftp.UploadDirectoryAsync(folderBrowser.SelectedPath, d.Name);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Download a directory
        /// </summary>
        private void btnDownloadDir_Click(object sender, EventArgs e)
        {
            try
            {
                if (treeDir.SelectedNode == null)
                {
                    SetCommandGui(true);
                    MessageBox.Show("Please select a directory to download!");
                    return;
                }

                if (folderBrowser.ShowDialog() == DialogResult.OK)
                {
                    SetCommandGui(false);
                    _sftp.DownloadDirectoryAsync(folderBrowser.SelectedPath,treeDir.SelectedNode.Text);
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        /// <summary>
        /// Cancel a transfer
        /// </summary>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            try
            {
                _sftp.Cancel();
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        private void btnPrivateKeyFile_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                txtPrivateKeyFile.Text = openFileDialog.FileName;
            }
        }
        #endregion

    }
}